"""
SPDX-License-Identifier: MIT

  Copyright (c) 2024, SCANOSS

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
"""

import os
import sys
from pathlib import Path
from typing import List, Optional

from pathspec import GitIgnoreSpec

from .scanossbase import ScanossBase

# Files to skip
DEFAULT_SKIPPED_FILES = {
    'gradlew',
    'gradlew.bat',
    'mvnw',
    'mvnw.cmd',
    'gradle-wrapper.jar',
    'maven-wrapper.jar',
    'thumbs.db',
    'babel.config.js',
    'license.txt',
    'license.md',
    'copying.lib',
    'makefile',
}

DEFAULT_SKIPPED_FILES_HFH = {
    'gradlew',
    'gradlew.bat',
    'mvnw',
    'mvnw.cmd',
    'gradle-wrapper.jar',
    'maven-wrapper.jar',
    'thumbs.db',
    'babel.config.js',
}


# Folders to skip
DEFAULT_SKIPPED_DIRS = {
    'nbproject',
    'nbbuild',
    'nbdist',
    '__pycache__',
    'venv',
    '_yardoc',
    'eggs',
    'wheels',
    'htmlcov',
    '__pypackages__',
    'example',
    'examples'
}

DEFAULT_SKIPPED_DIRS_HFH = {
    'nbproject',
    'nbbuild',
    'nbdist',
    '__pycache__',
    'venv',
    '_yardoc',
    'eggs',
    'wheels',
    'htmlcov',
    '__pypackages__',
    'example',
    'examples',
}


# Folder endings to skip
DEFAULT_SKIPPED_DIR_EXT = {'.egg-info'}
DEFAULT_SKIPPED_DIR_EXT_HFH = {'.egg-info'}

# File extensions to skip
DEFAULT_SKIPPED_EXT = {
    '.1',
    '.2',
    '.3',
    '.4',
    '.5',
    '.6',
    '.7',
    '.8',
    '.9',
    '.ac',
    '.adoc',
    '.am',
    '.asciidoc',
    '.bmp',
    '.build',
    '.cfg',
    '.chm',
    '.class',
    '.cmake',
    '.cnf',
    '.conf',
    '.config',
    '.contributors',
    '.copying',
    '.crt',
    '.csproj',
    '.css',
    '.csv',
    '.dat',
    '.data',
    '.doc',
    '.docx',
    '.dtd',
    '.dts',
    '.iws',
    '.c9',
    '.c9revisions',
    '.dtsi',
    '.dump',
    '.eot',
    '.eps',
    '.geojson',
    '.gdoc',
    '.gif',
    '.glif',
    '.gmo',
    '.gradle',
    '.guess',
    '.hex',
    '.htm',
    '.html',
    '.ico',
    '.iml',
    '.in',
    '.inc',
    '.info',
    '.ini',
    '.ipynb',
    '.jpeg',
    '.jpg',
    '.json',
    '.jsonld',
    '.lock',
    '.log',
    '.m4',
    '.map',
    '.markdown',
    '.md',
    '.md5',
    '.meta',
    '.mk',
    '.mxml',
    '.o',
    '.otf',
    '.out',
    '.pbtxt',
    '.pdf',
    '.pem',
    '.phtml',
    '.plist',
    '.png',
    '.po',
    '.ppt',
    '.prefs',
    '.properties',
    '.pyc',
    '.qdoc',
    '.result',
    '.rgb',
    '.rst',
    '.scss',
    '.sha',
    '.sha1',
    '.sha2',
    '.sha256',
    '.sln',
    '.spec',
    '.sql',
    '.sub',
    '.svg',
    '.svn-base',
    '.tab',
    '.template',
    '.test',
    '.tex',
    '.tiff',
    '.toml',
    '.ttf',
    '.txt',
    '.utf-8',
    '.vim',
    '.wav',
    '.woff',
    '.woff2',
    '.xht',
    '.xhtml',
    '.xls',
    '.xlsx',
    '.xml',
    '.xpm',
    '.xsd',
    '.xul',
    '.yaml',
    '.yml',
    '.wfp',
    '.editorconfig',
    '.dotcover',
    '.pid',
    '.lcov',
    '.egg',
    '.manifest',
    '.cache',
    '.coverage',
    '.cover',
    '.gem',
    '.lst',
    '.pickle',
    '.pdb',
    '.gml',
    '.pot',
    '.plt',
    '.whml',
    '.pom',
    '.smtml',
    '.min.js',
    '.mf',
    '.base64',
    '.s',
    '.diff',
    '.patch',
    '.rules',
    # File endings
    '-doc',
    'changelog',
    'config',
    'copying',
    'license',
    'authors',
    'news',
    'licenses',
    'notice',
    'readme',
    'swiftdoc',
    'texidoc',
    'todo',
    'version',
    'ignore',
    'manifest',
    'sqlite',
    'sqlite3',
}

# TODO: For hfh add the .gitignore patterns
DEFAULT_SKIPPED_EXT_HFH = {
    '.1',
    '.2',
    '.3',
    '.4',
    '.5',
    '.6',
    '.7',
    '.8',
    '.9',
    '.ac',
    '.adoc',
    '.am',
    '.asciidoc',
    '.bmp',
    '.build',
    '.cfg',
    '.chm',
    '.class',
    '.cmake',
    '.cnf',
    '.conf',
    '.config',
    '.contributors',
    '.copying',
    '.crt',
    '.csproj',
    '.css',
    '.csv',
    '.dat',
    '.data',
    '.dtd',
    '.dts',
    '.iws',
    '.c9',
    '.c9revisions',
    '.dtsi',
    '.dump',
    '.eot',
    '.eps',
    '.geojson',
    '.gif',
    '.glif',
    '.gmo',
    '.guess',
    '.hex',
    '.htm',
    '.html',
    '.ico',
    '.iml',
    '.in',
    '.inc',
    '.info',
    '.ini',
    '.ipynb',
    '.jpeg',
    '.jpg',
    '.json',
    '.jsonld',
    '.lock',
    '.log',
    '.m4',
    '.map',
    '.md5',
    '.meta',
    '.mk',
    '.mxml',
    '.o',
    '.otf',
    '.out',
    '.pbtxt',
    '.pdf',
    '.pem',
    '.phtml',
    '.plist',
    '.png',
    '.prefs',
    '.properties',
    '.pyc',
    '.qdoc',
    '.result',
    '.rgb',
    '.rst',
    '.scss',
    '.sha',
    '.sha1',
    '.sha2',
    '.sha256',
    '.sln',
    '.spec',
    '.sub',
    '.svg',
    '.svn-base',
    '.tab',
    '.template',
    '.test',
    '.tex',
    '.tiff',
    '.ttf',
    '.txt',
    '.utf-8',
    '.vim',
    '.wav',
    '.woff',
    '.woff2',
    '.xht',
    '.xhtml',
    '.xml',
    '.xpm',
    '.xsd',
    '.xul',
    '.yaml',
    '.yml',
    '.wfp',
    '.editorconfig',
    '.dotcover',
    '.pid',
    '.lcov',
    '.egg',
    '.manifest',
    '.cache',
    '.coverage',
    '.cover',
    '.gem',
    '.lst',
    '.pickle',
    '.pdb',
    '.gml',
    '.pot',
    '.plt',
    '.whml',
    '.pom',
    '.smtml',
    '.min.js',
    '.mf',
    '.base64',
    '.s',
    '.diff',
    '.patch',
    '.rules',
    # File endings
    '-doc',
    'config',
    'news',
    'readme',
    'swiftdoc',
    'texidoc',
    'todo',
    'version',
    'ignore',
    'manifest',
    'sqlite',
    'sqlite3',
}


class FileFilters(ScanossBase):
    """
    Filter for determining which files to process during scanning, fingerprinting, etc.
    Handles both inclusion and exclusion rules based on file paths, extensions, and sizes.
    """

    def __init__(self, debug: bool = False, trace: bool = False, quiet: bool = False, **kwargs):
        """
        Initialize scan filters based on default settings. Optionally append custom settings.

        Args:
            debug (bool): Enable debug output
            trace (bool): Enable trace output
            quiet (bool): Suppress output
            **kwargs: Additional arguments including:
                scanoss_settings (ScanossSettings): Custom settings to override defaults
                all_extensions (bool): Include all file extensions
                all_folders (bool): Include all folders
                hidden_files_folders (bool): Include hidden files and folders
                operation_type (str): Operation type ('scanning' or 'fingerprinting')
                skip_size (int): Size to skip
                skip_extensions (list): Extensions to skip
                skip_folders (list): Folders to skip
                is_folder_hashing_scan (bool): Whether the operation is a folder hashing scan
        """
        super().__init__(debug, trace, quiet)

        self.hidden_files_folders = kwargs.get('hidden_files_folders', False)
        self.scanoss_settings = kwargs.get('scanoss_settings')
        self.all_extensions = kwargs.get('all_extensions', False)
        self.all_folders = kwargs.get('all_folders', False)
        self.skip_folders = kwargs.get('skip_folders', [])
        self.skip_size = kwargs.get('skip_size', 0)
        self.skip_extensions = kwargs.get('skip_extensions', [])
        self.is_folder_hashing_scan = kwargs.get('is_folder_hashing_scan', False)
        self.file_folder_pat_spec = self._get_file_folder_pattern_spec(kwargs.get('operation_type', 'scanning'))
        self.size_pat_rules = self._get_size_limit_pattern_rules(kwargs.get('operation_type', 'scanning'))

    def get_filtered_files_from_folder(self, root: str) -> List[str]:
        """
        Retrieve a list of files to scan or fingerprint from a given directory root based on filter settings.

        Args:
            root (str): Root directory to scan or fingerprint

        Returns:
            list[str]: Filtered list of files to scan or fingerprint
        """
        if self.debug:
            if self.file_folder_pat_spec:
                self.print_stderr(f'Running with {len(self.file_folder_pat_spec)} pattern filters.')
            if self.size_pat_rules:
                self.print_stderr(f'Running with {len(self.size_pat_rules)} size pattern rules.')
            if self.skip_size:
                self.print_stderr(f'Running with global skip size: {self.skip_size}')
            if self.skip_extensions:
                self.print_stderr(f'Running with extra global skip extensions: {self.skip_extensions}')
            if self.skip_folders:
                self.print_stderr(f'Running with extra global skip folders: {self.skip_folders}')
        all_files = []
        root_path = Path(root).resolve()
        if not root_path.exists() or not root_path.is_dir():
            self.print_stderr(f'ERROR: Specified root directory {root} does not exist or is not a directory.')
            return all_files
        # Walk the tree looking for files to process. While taking into account files/folders to skip
        for dirpath, dirnames, filenames in os.walk(root_path):
            dir_path = Path(dirpath)
            rel_path = dir_path.relative_to(root_path)
            if dir_path.is_symlink():  # TODO should we skip symlink folders?
                self.print_msg(f'WARNING: Found symbolic link folder: {dir_path}')

            if self.should_skip_dir(str(rel_path)):  # Current directory should be skipped
                dirnames.clear()
                continue
            for filename in filenames:
                file_path = dir_path / filename
                all_files.append(str(file_path))
        # End os.walk loop
        # Now filter the files and return the reduced list
        return self.get_filtered_files_from_files(all_files, str(root_path))

    def get_filtered_files_from_files(self, files: List[str], scan_root: Optional[str] = None) -> List[str]:
        """
        Retrieve a list of files to scan or fingerprint from a given list of files based on filter settings.

        Args:
            files (List[str]): List of files to scan or fingerprint
            scan_root (str): Root directory to scan or fingerprint

        Returns:
            list[str]: Filtered list of files to scan or fingerprint
        """
        filtered_files = []
        for file_path in files:
            path_obj = Path(file_path)
            try:
                if scan_root:
                    rel_path = path_obj.relative_to(scan_root)
                else:
                    rel_path = str(path_obj)
            except ValueError:
                self.print_debug(f'Ignoring file: {file_path} (broken symlink)')
                continue

            if not path_obj.exists() or not path_obj.is_file() or path_obj.is_symlink():
                self.print_debug(
                    f'WARNING: File {rel_path} does not exist, is not a file, or is a symbolic link. Ignoring.'
                )
                continue

            if not self.hidden_files_folders and any(part.startswith('.') for part in path_obj.parts):
                self.print_debug(f'Skipping file: {rel_path} (in hidden directory or is hidden file)')
                continue

            if self._should_skip_file(rel_path):
                continue
            try:
                file_size = path_obj.stat().st_size
                if file_size == 0:
                    self.print_debug(f'Skipping file: {rel_path} (empty file)')
                    continue
                min_size, max_size = self._get_operation_size_limits(file_path)
                if min_size <= file_size <= max_size:
                    filtered_files.append(str(rel_path))
                else:
                    self.print_debug(
                        f'Skipping file: {rel_path} (size {file_size} outside limits {min_size}-{max_size})'
                    )
            except OSError as e:
                self.print_debug(f'Error getting size for {rel_path}: {e}')
        # End file loop
        return filtered_files

    def _get_file_folder_pattern_spec(self, operation_type: str = 'scanning'):
        """
        Get file path pattern specification.

        Args:
            operation_type (str): Type of operation ('scanning' or 'fingerprinting')

        Returns:
            GitIgnoreSpec: GitIgnoreSpec object containing the file path patterns
        """
        patterns = self._get_operation_patterns(operation_type)
        if patterns:
            return GitIgnoreSpec.from_lines(patterns)
        return None

    def _get_size_limit_pattern_rules(self, operation_type: str = 'scanning'):
        """
        Get size limit pattern rules.

        Args:
            operation_type (str): Type of operation ('scanning' or 'fingerprinting')

        Returns:
            List of size limit pattern rules
        """
        if self.scanoss_settings:
            size_rules = self.scanoss_settings.get_skip_sizes(operation_type)
            if size_rules:
                size_rules_with_patterns = []
                for rule in size_rules:
                    patterns = rule.get('patterns', [])
                    if not patterns:
                        continue
                    size_rules_with_patterns.append(rule)
                return size_rules_with_patterns
        return None

    def _get_operation_patterns(self, operation_type: str) -> List[str]:
        """
        Get patterns specific to the operation type, combining defaults with settings.

        Args:
            operation_type (str): Type of operation ('scanning' or 'fingerprinting')

        Returns:
            List[str]: Combined list of patterns to skip
        """
        patterns = []

        # Default patterns for skipping directories
        if not self.all_folders:
            DEFAULT_SKIPPED_DIR_LIST = DEFAULT_SKIPPED_DIRS_HFH if self.is_folder_hashing_scan else DEFAULT_SKIPPED_DIRS
            DEFAULT_SKIPPED_DIR_EXT_LIST = (
                DEFAULT_SKIPPED_DIR_EXT_HFH if self.is_folder_hashing_scan else DEFAULT_SKIPPED_DIR_EXT
            )
            for dir_name in DEFAULT_SKIPPED_DIR_LIST:
                patterns.append(f'{dir_name}/')
            for dir_extension in DEFAULT_SKIPPED_DIR_EXT_LIST:
                patterns.append(f'*{dir_extension}/')

        # Custom patterns added in SCANOSS settings file
        if self.scanoss_settings:
            patterns.extend(self.scanoss_settings.get_skip_patterns(operation_type))
        return patterns

    def _get_operation_size_limits(self, file_path: str = None) -> tuple:
        """
        Get size limits specific to the operation type and file path.

        Args:
            file_path (str, optional): Path to the file to check against patterns. If None, returns default limits.

        Returns:
            tuple: (min_size, max_size) tuple for the given file path and operation type
        """
        min_size = 0
        max_size = sys.maxsize
        # Apply global minimum file size if specified
        if self.skip_size > 0:
            min_size = self.skip_size
            return min_size, max_size
        # Return default size limits if no settings specified
        if not self.scanoss_settings or not file_path or not self.size_pat_rules:
            return min_size, max_size
        try:
            rel_path = os.path.relpath(file_path)
        except ValueError:
            rel_path = os.path.basename(file_path)
        rel_path_lower = rel_path.lower()
        # Cycle through each rule looking for a match
        for rule in self.size_pat_rules:
            patterns = rule.get('patterns', [])
            if patterns:
                path_spec = GitIgnoreSpec.from_lines(patterns)
                if path_spec.match_file(rel_path_lower):
                    return rule.get('min', min_size), rule.get('max', max_size)
        # End rules loop
        return min_size, max_size

    def should_skip_dir(self, dir_rel_path: str) -> bool:  # noqa: PLR0911
        """
        Check if a directory should be skipped based on operation type and default rules.

        Args:
            dir_rel_path (str): Relative path to the directory

        Returns:
            bool: True if directory should be skipped, False otherwise
        """
        dir_name = os.path.basename(dir_rel_path)
        dir_path = Path(dir_rel_path)
        if (
            not self.hidden_files_folders
            and dir_path != Path('.')
            and any(part.startswith('.') for part in dir_path.parts)
        ):
            self.print_debug(f'Skipping directory: {dir_rel_path} (hidden directory)')
            return True
        if self.all_folders:
            return False
        dir_name_lower = dir_name.lower()
        if dir_name_lower in DEFAULT_SKIPPED_DIRS:
            self.print_debug(f'Skipping directory: {dir_rel_path} (matches default skip directory)')
            return True
        if self.skip_folders and dir_name in self.skip_folders:
            self.print_debug(f'Skipping directory: {dir_rel_path} (matches skip folder)')
            return True
        for ext in DEFAULT_SKIPPED_DIR_EXT:
            if dir_name_lower.endswith(ext):
                self.print_debug(f'Skipping directory: {dir_rel_path} (matches default skip extension: {ext})')
                return True

        if self.file_folder_pat_spec and self.file_folder_pat_spec.match_file(dir_rel_path):
            self.print_debug(f'Skipping directory: {dir_rel_path} (matches custom pattern)')
            return True
        return False

    def _should_skip_file(self, file_rel_path: str) -> bool:  # noqa: PLR0911
        """
        Check if a file should be skipped based on operation type and default rules.

        Args:
            file_rel_path (str): Relative path to the file

        Returns:
            bool: True if file should be skipped, False otherwise
        """
        file_name = os.path.basename(file_rel_path)

        DEFAULT_SKIPPED_FILES_LIST = DEFAULT_SKIPPED_FILES_HFH if self.is_folder_hashing_scan else DEFAULT_SKIPPED_FILES
        DEFAULT_SKIPPED_EXT_LIST = DEFAULT_SKIPPED_EXT_HFH if self.is_folder_hashing_scan else DEFAULT_SKIPPED_EXT

        if not self.hidden_files_folders and file_name.startswith('.'):
            self.print_debug(f'Skipping file: {file_rel_path} (hidden file)')
            return True
        if self.all_extensions:
            return False
        file_name_lower = file_name.lower()
        # Look for exact files
        if file_name_lower in DEFAULT_SKIPPED_FILES_LIST:
            self.print_debug(f'Skipping file: {file_rel_path} (matches default skip file)')
            return True
        # Look for file endings
        for ending in DEFAULT_SKIPPED_EXT_LIST:
            if file_name_lower.endswith(ending):
                self.print_debug(f'Skipping file: {file_rel_path} (matches default skip ending: {ending})')
                return True
        # Look for custom (extra) endings
        if self.skip_extensions:
            for ending in self.skip_extensions:
                if file_name_lower.endswith(ending):
                    self.print_debug(f'Skipping file: {file_rel_path} (matches skip extension)')
                    return True
        # Check for file patterns
        if self.file_folder_pat_spec and self.file_folder_pat_spec.match_file(file_rel_path):
            self.print_debug(f'Skipping file: {file_rel_path} (matches custom pattern)')
            return True
        return False



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from scanoss.api.common.v2 import scanoss_common_pb2 as scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2
from scanoss.api.vulnerabilities.v2 import scanoss_vulnerabilities_pb2 as scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2

GRPC_GENERATED_VERSION = '1.73.1'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in scanoss/api/vulnerabilities/v2/scanoss_vulnerabilities_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class VulnerabilitiesStub(object):
    """
    Vulnerability Service Definition
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Echo = channel.unary_unary(
                '/scanoss.api.vulnerabilities.v2.Vulnerabilities/Echo',
                request_serializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.EchoRequest.SerializeToString,
                response_deserializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.EchoResponse.FromString,
                _registered_method=True)
        self.GetCpes = channel.unary_unary(
                '/scanoss.api.vulnerabilities.v2.Vulnerabilities/GetCpes',
                request_serializer=scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.VulnerabilityRequest.SerializeToString,
                response_deserializer=scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.CpeResponse.FromString,
                _registered_method=True)
        self.GetComponentCpes = channel.unary_unary(
                '/scanoss.api.vulnerabilities.v2.Vulnerabilities/GetComponentCpes',
                request_serializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.ComponentRequest.SerializeToString,
                response_deserializer=scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.ComponentCpesResponse.FromString,
                _registered_method=True)
        self.GetComponentsCpes = channel.unary_unary(
                '/scanoss.api.vulnerabilities.v2.Vulnerabilities/GetComponentsCpes',
                request_serializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.ComponentsRequest.SerializeToString,
                response_deserializer=scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.ComponentsCpesResponse.FromString,
                _registered_method=True)
        self.GetVulnerabilities = channel.unary_unary(
                '/scanoss.api.vulnerabilities.v2.Vulnerabilities/GetVulnerabilities',
                request_serializer=scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.VulnerabilityRequest.SerializeToString,
                response_deserializer=scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.VulnerabilityResponse.FromString,
                _registered_method=True)
        self.GetComponentVulnerabilities = channel.unary_unary(
                '/scanoss.api.vulnerabilities.v2.Vulnerabilities/GetComponentVulnerabilities',
                request_serializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.ComponentRequest.SerializeToString,
                response_deserializer=scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.ComponentVulnerabilityResponse.FromString,
                _registered_method=True)
        self.GetComponentsVulnerabilities = channel.unary_unary(
                '/scanoss.api.vulnerabilities.v2.Vulnerabilities/GetComponentsVulnerabilities',
                request_serializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.ComponentsRequest.SerializeToString,
                response_deserializer=scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.ComponentsVulnerabilityResponse.FromString,
                _registered_method=True)


class VulnerabilitiesServicer(object):
    """
    Vulnerability Service Definition
    """

    def Echo(self, request, context):
        """
        Returns the same message that was sent, used for health checks and connectivity testing
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCpes(self, request, context):
        """
        Get CPEs (Common Platform Enumeration) associated with a PURL - legacy endpoint.

        Legacy method for retrieving Common Platform Enumeration identifiers
        associated with software components. Use GetComponentCpes instead.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetComponentCpes(self, request, context):
        """
        Get CPEs (Common Platform Enumeration) associated with a single software component.

        Returns Common Platform Enumeration identifiers that match the specified component.
        CPEs are used to identify IT platforms in vulnerability databases and enable
        vulnerability scanning and assessment.

        See: https://github.com/scanoss/papi/blob/main/protobuf/scanoss/api/vulnerabilities/v2/README.md?tab=readme-ov-file#getcomponentcpes
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetComponentsCpes(self, request, context):
        """
        Get CPEs (Common Platform Enumeration) associated with multiple software components.

        Returns Common Platform Enumeration identifiers for multiple components in a single request.
        CPEs are used to identify IT platforms in vulnerability databases and enable
        vulnerability scanning and assessment.

        See: https://github.com/scanoss/papi/blob/main/protobuf/scanoss/api/vulnerabilities/v2/README.md?tab=readme-ov-file#getcomponentscpes
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetVulnerabilities(self, request, context):
        """
        Get vulnerability details - legacy endpoint.

        Legacy method for retrieving vulnerability information for software components.
        Use GetComponentVulnerabilities or GetComponentsVulnerabilities instead.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetComponentVulnerabilities(self, request, context):
        """
        Get vulnerability information for a single software component.

        Analyzes the component and returns known vulnerabilities including CVE details,
        severity scores, publication dates, and other security metadata.
        Vulnerability data is sourced from various security databases and feeds.

        See: https://github.com/scanoss/papi/blob/main/protobuf/scanoss/api/vulnerabilities/v2/README.md?tab=readme-ov-file#getcomponentvulnerabilities
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetComponentsVulnerabilities(self, request, context):
        """
        Get vulnerability information for multiple software components in a single request.

        Analyzes multiple components and returns known vulnerabilities for each including CVE details,
        severity scores, publication dates, and other security metadata.
        Vulnerability data is sourced from various security databases and feeds.

        See: https://github.com/scanoss/papi/blob/main/protobuf/scanoss/api/vulnerabilities/v2/README.md?tab=readme-ov-file#getcomponentsvulnerabilities
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_VulnerabilitiesServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Echo': grpc.unary_unary_rpc_method_handler(
                    servicer.Echo,
                    request_deserializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.EchoRequest.FromString,
                    response_serializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.EchoResponse.SerializeToString,
            ),
            'GetCpes': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCpes,
                    request_deserializer=scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.VulnerabilityRequest.FromString,
                    response_serializer=scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.CpeResponse.SerializeToString,
            ),
            'GetComponentCpes': grpc.unary_unary_rpc_method_handler(
                    servicer.GetComponentCpes,
                    request_deserializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.ComponentRequest.FromString,
                    response_serializer=scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.ComponentCpesResponse.SerializeToString,
            ),
            'GetComponentsCpes': grpc.unary_unary_rpc_method_handler(
                    servicer.GetComponentsCpes,
                    request_deserializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.ComponentsRequest.FromString,
                    response_serializer=scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.ComponentsCpesResponse.SerializeToString,
            ),
            'GetVulnerabilities': grpc.unary_unary_rpc_method_handler(
                    servicer.GetVulnerabilities,
                    request_deserializer=scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.VulnerabilityRequest.FromString,
                    response_serializer=scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.VulnerabilityResponse.SerializeToString,
            ),
            'GetComponentVulnerabilities': grpc.unary_unary_rpc_method_handler(
                    servicer.GetComponentVulnerabilities,
                    request_deserializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.ComponentRequest.FromString,
                    response_serializer=scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.ComponentVulnerabilityResponse.SerializeToString,
            ),
            'GetComponentsVulnerabilities': grpc.unary_unary_rpc_method_handler(
                    servicer.GetComponentsVulnerabilities,
                    request_deserializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.ComponentsRequest.FromString,
                    response_serializer=scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.ComponentsVulnerabilityResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'scanoss.api.vulnerabilities.v2.Vulnerabilities', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('scanoss.api.vulnerabilities.v2.Vulnerabilities', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class Vulnerabilities(object):
    """
    Vulnerability Service Definition
    """

    @staticmethod
    def Echo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/scanoss.api.vulnerabilities.v2.Vulnerabilities/Echo',
            scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.EchoRequest.SerializeToString,
            scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.EchoResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetCpes(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/scanoss.api.vulnerabilities.v2.Vulnerabilities/GetCpes',
            scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.VulnerabilityRequest.SerializeToString,
            scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.CpeResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetComponentCpes(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/scanoss.api.vulnerabilities.v2.Vulnerabilities/GetComponentCpes',
            scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.ComponentRequest.SerializeToString,
            scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.ComponentCpesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetComponentsCpes(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/scanoss.api.vulnerabilities.v2.Vulnerabilities/GetComponentsCpes',
            scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.ComponentsRequest.SerializeToString,
            scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.ComponentsCpesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetVulnerabilities(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/scanoss.api.vulnerabilities.v2.Vulnerabilities/GetVulnerabilities',
            scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.VulnerabilityRequest.SerializeToString,
            scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.VulnerabilityResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetComponentVulnerabilities(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/scanoss.api.vulnerabilities.v2.Vulnerabilities/GetComponentVulnerabilities',
            scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.ComponentRequest.SerializeToString,
            scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.ComponentVulnerabilityResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetComponentsVulnerabilities(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/scanoss.api.vulnerabilities.v2.Vulnerabilities/GetComponentsVulnerabilities',
            scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.ComponentsRequest.SerializeToString,
            scanoss_dot_api_dot_vulnerabilities_dot_v2_dot_scanoss__vulnerabilities__pb2.ComponentsVulnerabilityResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



import json
import os
from dataclasses import dataclass
from pathlib import Path
from typing import Dict, List, Literal, Optional

from progress.bar import Bar

from scanoss.file_filters import FileFilters
from scanoss.scanoss_settings import ScanossSettings
from scanoss.scanossbase import ScanossBase
from scanoss.utils.abstract_presenter import AbstractPresenter
from scanoss.utils.crc64 import CRC64
from scanoss.utils.simhash import WordFeatureSet, fingerprint, simhash, vectorize_bytes

MINIMUM_FILE_COUNT = 8
MINIMUM_CONCATENATED_NAME_LENGTH = 32
MAXIMUM_FILE_NAME_LENGTH = 32


class DirectoryNode:
    """
    Represents a node in the directory tree for folder hashing.
    """

    def __init__(self, path: str):
        self.path = path
        self.is_dir = True
        self.children: Dict[str, DirectoryNode] = {}
        self.files: List[DirectoryFile] = []


class DirectoryFile:
    """
    Represents a file in the directory tree for folder hashing.
    """

    def __init__(self, path: str, key: List[bytes], key_str: str):
        self.path = path
        self.key = key
        self.key_str = key_str


@dataclass
class FolderHasherConfig:
    debug: bool = False
    trace: bool = False
    quiet: bool = False
    output_file: Optional[str] = None
    output_format: Literal['json'] = 'json'
    settings_file: Optional[str] = None
    skip_settings_file: bool = False


def create_folder_hasher_config_from_args(args) -> FolderHasherConfig:
    return FolderHasherConfig(
        debug=args.debug,
        trace=args.trace,
        quiet=args.quiet,
        output_file=getattr(args, 'output', None),
        output_format=getattr(args, 'format', 'json'),
        settings_file=getattr(args, 'settings', None),
        skip_settings_file=getattr(args, 'skip_settings_file', False),
    )


class FolderHasher:
    """
    Folder Hasher.

    This class is used to produce a folder hash for a given directory.

    It builds a directory tree (DirectoryNode) and computes the associated
    hash data for the folder.
    """

    def __init__(
        self,
        scan_dir: str,
        config: FolderHasherConfig,
        scanoss_settings: Optional[ScanossSettings] = None,
    ):
        self.base = ScanossBase(
            debug=config.debug,
            trace=config.trace,
            quiet=config.quiet,
        )
        self.file_filters = FileFilters(
            debug=config.debug,
            trace=config.trace,
            quiet=config.quiet,
            scanoss_settings=scanoss_settings,
            is_folder_hashing_scan=True,
        )
        self.presenter = FolderHasherPresenter(
            self,
            debug=config.debug,
            trace=config.trace,
            quiet=config.quiet,
        )

        self.scan_dir = scan_dir
        self.tree = None

    def hash_directory(self, path: str) -> dict:
        """
        Generate the folder hashing request structure from a directory path.

        This method builds a directory tree (DirectoryNode) and computes the associated
        hash data for the folder.

        Args:
            path (str): The root directory path.

        Returns:
            dict: The folder hash request structure.
        """

        root_node = self._build_root_node(path)
        tree = self._hash_calc_from_node(root_node)

        self.tree = tree

        return tree

    def _build_root_node(self, path: str) -> DirectoryNode:
        """
        Build a directory tree from the given path with file information.

        The tree includes DirectoryNode objects populated with filtered file items,
        each containing their relative path and CRC64 hash key.

        Args:
            path (str): The directory path to build the tree from.

        Returns:
            DirectoryNode: The root node representing the directory.
        """
        root = Path(path).resolve()
        root_node = DirectoryNode(str(root))

        all_files = [
            f for f in root.rglob('*') if f.is_file() and len(f.name.encode('utf-8')) <= MAXIMUM_FILE_NAME_LENGTH
        ]
        filtered_files = self.file_filters.get_filtered_files_from_files(all_files, str(root))

        # Sort the files by name to ensure the hash is the same for the same folder
        filtered_files.sort()

        bar = Bar('Hashing files...', max=len(filtered_files))
        for file_path in filtered_files:
            try:
                file_path_obj = Path(file_path) if isinstance(file_path, str) else file_path
                full_file_path = file_path_obj if file_path_obj.is_absolute() else root / file_path_obj

                self.base.print_debug(f'\nHashing file {str(full_file_path)}')

                file_bytes = full_file_path.read_bytes()
                key = CRC64.get_hash_buff(file_bytes)
                key_str = ''.join(f'{b:02x}' for b in key)
                rel_path = str(full_file_path.relative_to(root))

                file_item = DirectoryFile(rel_path, key, key_str)

                current_node = root_node
                for part in Path(rel_path).parent.parts:
                    child_path = str(Path(current_node.path) / part)
                    if child_path not in current_node.children:
                        current_node.children[child_path] = DirectoryNode(child_path)
                    current_node = current_node.children[child_path]
                    current_node.files.append(file_item)

                root_node.files.append(file_item)

            except Exception as e:
                self.base.print_debug(f'Skipping file {full_file_path}: {str(e)}')

            bar.next()

        bar.finish()
        return root_node

    def _hash_calc_from_node(self, node: DirectoryNode) -> dict:
        """
        Recursively compute folder hash data for a directory node.

        The hash data includes the path identifier, simhash for file names,
        simhash for file content, directory hash, language extensions, and children node hash information.

        Args:
            node (DirectoryNode): The directory node to compute the hash for.

        Returns:
            dict: The computed hash data for the node.
        """
        hash_data = self._hash_calc(node)
        
        # Safely calculate relative path
        try:
            node_path = Path(node.path).resolve()
            scan_dir_path = Path(self.scan_dir).resolve()
            rel_path = node_path.relative_to(scan_dir_path)
        except ValueError:
            # If relative_to fails, use the node path as is or a fallback
            rel_path = Path(node.path).name if node.path else Path('.')

        return {
            'path_id': str(rel_path),
            'sim_hash_names': f'{hash_data["name_hash"]:02x}' if hash_data['name_hash'] is not None else None,
            'sim_hash_content': f'{hash_data["content_hash"]:02x}' if hash_data['content_hash'] is not None else None,
            'sim_hash_dir_names': f'{hash_data["dir_hash"]:02x}' if hash_data['dir_hash'] is not None else None,
            'lang_extensions': hash_data['lang_extensions'],
            'children': [self._hash_calc_from_node(child) for child in node.children.values()],
        }

    def _hash_calc(self, node: DirectoryNode) -> dict:
        """
        Compute folder hash values for a given directory node.

        The method aggregates unique file keys and sorted file names to generate
        simhash-based hash values for both file names and file contents.

        The most significant byte of the name simhash is then replaced by a computed head value.

        Args:
            node (DirectoryNode): The directory node containing file items.

        Returns:
            dict: A dictionary with 'name_hash', 'content_hash', 'dir_hash', and 'lang_extensions' keys.
        """
        processed_hashes = set()
        unique_file_names = set()
        unique_directories = set()
        extension_map = {}
        file_hashes = []
        selected_names = []

        for file in node.files:
            key_str = file.key_str
            if key_str in processed_hashes:
                continue

            file_name = os.path.basename(file.path)

            file_name_without_extension, extension = os.path.splitext(file_name)
            current_directory = os.path.dirname(file.path)

            if extension and len(extension) > 1:
                ext_without_dot = extension[1:]
                extension_map[ext_without_dot] = extension_map.get(ext_without_dot, 0) + 1

            current_directory.replace(self.scan_dir, '', 1).lstrip(os.path.sep)
            parts = current_directory.split(os.path.sep)
            for d in parts:
                if d in {'', '.', '..'}:
                    continue
                unique_directories.add(d)

            processed_hashes.add(key_str)
            unique_file_names.add(file_name_without_extension)
            selected_names.append(file_name)
            file_hashes.append(file.key)

        if len(selected_names) < MINIMUM_FILE_COUNT:
            return {'name_hash': None, 'content_hash': None, 'dir_hash': None, 'lang_extensions': None}

        selected_names.sort()
        concatenated_names = ''.join(selected_names)

        if len(concatenated_names.encode('utf-8')) < MINIMUM_CONCATENATED_NAME_LENGTH:
            return {'name_hash': None, 'content_hash': None, 'dir_hash': None, 'lang_extensions': None}

        # Concatenate the unique file names without the extensions, adding a space and sorting them alphabetically
        unique_file_names_list = list(unique_file_names)
        unique_file_names_list.sort()
        concatenated_names = ' '.join(unique_file_names_list)

        # We do the same for the directory names, adding a space and sorting them alphabetically
        unique_directories_list = list(unique_directories)
        unique_directories_list.sort()
        concatenated_directories = ' '.join(unique_directories_list)

        names_simhash = simhash(WordFeatureSet(concatenated_names.encode('utf-8')))
        dir_simhash = simhash(WordFeatureSet(concatenated_directories.encode('utf-8')))
        content_simhash = fingerprint(vectorize_bytes(file_hashes))

        # Debug logging similar to Go implementation
        self.base.print_debug(f'Unique file names: {unique_file_names_list}')
        self.base.print_debug(f'Unique directories: {unique_directories_list}')
        self.base.print_debug(f'{dir_simhash:x}/{names_simhash:x} - {content_simhash:x} - {extension_map}')

        return {
            'name_hash': names_simhash,
            'content_hash': content_simhash,
            'dir_hash': dir_simhash,
            'lang_extensions': extension_map,
        }

    def present(self, output_format: Optional[str] = None, output_file: Optional[str] = None):
        """Present the hashed tree in the selected format"""
        self.presenter.present(output_format=output_format, output_file=output_file)


class FolderHasherPresenter(AbstractPresenter):
    """
    FolderHasher presenter class
    Handles the presentation of the folder hashing scan results
    """

    def __init__(self, folder_hasher: FolderHasher, **kwargs):
        super().__init__(**kwargs)
        self.folder_hasher = folder_hasher

    def _format_json_output(self) -> str:
        """
        Format the scan output data into a JSON object

        Returns:
            str: The formatted JSON string
        """
        return json.dumps(self.folder_hasher.tree, indent=2)

    def _format_plain_output(self) -> str:
        """
        Format the scan output data into a plain text string
        """
        return (
            json.dumps(self.folder_hasher.tree, indent=2)
            if isinstance(self.folder_hasher.tree, dict)
            else str(self.folder_hasher.tree)
        )

    def _format_cyclonedx_output(self) -> str:
        raise NotImplementedError('CycloneDX output is not implemented')

    def _format_spdxlite_output(self) -> str:
        raise NotImplementedError('SPDXlite output is not implemented')

    def _format_csv_output(self) -> str:
        raise NotImplementedError('CSV output is not implemented')

    def _format_raw_output(self) -> str:
        raise NotImplementedError('Raw output is not implemented')



"""
SPDX-License-Identifier: MIT

  Copyright (c) 2025, SCANOSS

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
"""

import json
from dataclasses import dataclass
from typing import Any, Dict, List

from ..policy_check import PolicyStatus
from .raw_base import RawBase


@dataclass
class License:
    spdxid: str
    copyleft: bool
    url: str

@dataclass
class Component:
    purl: str
    version: str
    licenses: List[License]
    status: str

class UndeclaredComponent(RawBase[Component]):
    """
    SCANOSS UndeclaredComponent class
    Inspects for undeclared components
    """

    def __init__( # noqa: PLR0913
        self,
        debug: bool = False,
        trace: bool = False,
        quiet: bool = False,
        filepath: str = None,
        format_type: str = 'json',
        status: str = None,
        output: str = None,
        sbom_format: str = 'settings',
    ):
        """
        Initialize the UndeclaredComponent class.

        :param debug: Enable debug mode
        :param trace: Enable trace mode (default True)
        :param quiet: Enable quiet mode
        :param filepath: Path to the file containing component data
        :param format_type: Output format ('json' or 'md')
        :param status: Path to save status output
        :param output: Path to save detailed output
        :param sbom_format: Sbom format for status output (default 'settings')
        """
        super().__init__(
            debug, trace, quiet,format_type, filepath, output, status, name='Undeclared Components Policy'
        )
        self.filepath = filepath
        self.format = format
        self.output = output
        self.status = status
        self.sbom_format = sbom_format

    def _get_undeclared_components(self, components: list[Component]) -> list or None:
        """
        Filter the components list to include only undeclared components.

        :param components: List of all components
        :return: List of undeclared components
        """
        if components is None:
            self.print_debug('WARNING: No components provided!')
            return None
        undeclared_components = []
        for component in components:
            if component['status'] == 'pending':
                # Remove unused keys
                del component['count']
                del component['declared']
                del component['undeclared']
                for lic in component['licenses']:
                        lic.pop('count', None)  # None is default value if key doesn't exist
                        lic.pop('source', None) # None is default value if key doesn't exist
                undeclared_components.append(component)
        # end component loop
        return undeclared_components

    def _get_jira_summary(self, components: list[Component]) -> str:
        """
        Get a summary of the undeclared components.

        :param components: List of all components
        :return: Component summary markdown
        """

        """
        Get a summary of the undeclared components.

        :param components: List of all components
        :return: Component summary markdown
        """
        if len(components) > 0:
            json_content = json.dumps(self._generate_scanoss_file(components), indent=2)

            if self.sbom_format == 'settings':
                return (
                    f'{len(components)} undeclared component(s) were found.\n'
                    f'Add the following snippet into your `scanoss.json` file\n'
                    f'{{code:json}}\n'
                    f'{json_content}\n'
                    f'{{code}}\n'
                )
            else:
                return (
                    f'{len(components)} undeclared component(s) were found.\n'
                    f'Add the following snippet into your `sbom.json` file\n'
                    f'{{code:json}}\n'
                    f'{json_content}\n'
                    f'{{code}}\n'
                )
        return f'{len(components)} undeclared component(s) were found.\\n'

    def _get_summary(self, components: list) -> str:
        """
        Get a summary of the undeclared components.

        :param components: List of all components
        :return: Component summary markdown
        """
        summary = f'{len(components)} undeclared component(s) were found.\n'
        if len(components) > 0:
            if self.sbom_format == 'settings':
                summary += (
                    f'Add the following snippet into your `scanoss.json` file\n'
                    f'\n```json\n{json.dumps(self._generate_scanoss_file(components), indent=2)}\n```\n'
                )
            else:
                summary += (
                    f'Add the following snippet into your `sbom.json` file\n'
                    f'\n```json\n{json.dumps(self._generate_sbom_file(components), indent=2)}\n```\n'
                )

        return summary

    def _json(self, components: list[Component]) -> Dict[str, Any]:
        """
        Format the undeclared components as JSON.

        :param components: List of undeclared components
        :return: Dictionary with formatted JSON details and summary
        """
        # Use component grouped by licenses to generate the summary
        component_licenses = self._group_components_by_license(components)
        details = {}
        if len(components) > 0:
            details = {'components': components}
        return {
            'details': f'{json.dumps(details, indent=2)}\n',
            'summary': self._get_summary(component_licenses),
        }

    def _markdown(self, components: list[Component]) -> Dict[str, Any]:
        """
        Format the undeclared components as Markdown.

        :param components: List of undeclared components
        :return: Dictionary with formatted Markdown details and summary
        """
        headers = ['Component', 'License']
        rows = []
        # TODO look at using SpdxLite license name lookup method
        component_licenses = self._group_components_by_license(components)
        for component in component_licenses:
            rows.append([component.get('purl'), component.get('spdxid')])
        return {
            'details': f'### Undeclared components\n{self.generate_table(headers, rows)}\n',
            'summary': self._get_summary(component_licenses),
        }

    def _jira_markdown(self, components: list) -> Dict[str, Any]:
        """
        Format the undeclared components as Markdown.

        :param components: List of undeclared components
        :return: Dictionary with formatted Markdown details and summary
        """
        headers = ['Component', 'License']
        rows = []
        # TODO look at using SpdxLite license name lookup method
        component_licenses = self._group_components_by_license(components)
        for component in component_licenses:
            rows.append([component.get('purl'), component.get('spdxid')])
        return {
            'details': f'{self.generate_jira_table(headers, rows)}',
            'summary': self._get_jira_summary(component_licenses),
        }

    def _get_unique_components(self, components: list) -> list:
        """
        Generate a list of unique components.

        :param components: List of undeclared components
        :return: list of unique components
        """
        unique_components = {}
        if components is None:
            self.print_stderr('WARNING: No components provided!')
            return []

        for component in components:
            unique_components[component['purl']] = {'purl': component['purl']}
        return list(unique_components.values())

    def _generate_scanoss_file(self, components: list) -> dict:
        """
        Generate a list of PURLs for the scanoss.json file.

        :param components: List of undeclared components
        :return: scanoss.json Dictionary
        """
        scanoss_settings = {
            'bom': {
                'include': self._get_unique_components(components),
            }
        }

        return scanoss_settings

    def _generate_sbom_file(self, components: list) -> dict:
        """
        Generate a list of PURLs for the SBOM file.

        :param components: List of undeclared components
        :return: SBOM Dictionary with components
        """
        sbom = {
            'components': self._get_unique_components(components),
        }

        return sbom

    def _get_components(self):
        """
        Extract and process components from file results only.

        This method performs the following steps:
        1. Validates if `self.results` is loaded. Returns `None` if not loaded.
        2. Extracts file and snippet components into a dictionary.
        3. Converts the components dictionary into a list of components.
        4. Processes the licenses for each component by converting them into a list.

        :return: A list of processed components with their licenses, or `None` if `self.results` is not set.
        """
        if self.results is None:
            return None
        components: dict = {}
        # Extract file and snippet components
        components = self._get_components_data(self.results, components)
        # Convert to list and process licenses
        return self._convert_components_to_list(components)

    def run(self):
        """
        Run the undeclared component inspection process.

        This method performs the following steps:
        1. Get all components
        2. Filter undeclared components
        3. Format the results
        4. Save the output to files if required

        :return: Dictionary containing the inspection results
        """
        self._debug()
        components = self._get_components()
        if components is None:
            return PolicyStatus.ERROR.value, {}
        # Get an undeclared component summary (if any)
        undeclared_components = self._get_undeclared_components(components)
        if undeclared_components is None:
            return PolicyStatus.ERROR.value, {}
        self.print_debug(f'Undeclared components: {undeclared_components}')
        # Format the results and save to files if required
        return self._generate_formatter_report(undeclared_components)
#
# End of UndeclaredComponent Class
#



"""
SPDX-License-Identifier: MIT

  Copyright (c) 2023, SCANOSS

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
"""

import json
import os
import sys
from typing import List, Optional, TextIO

from pypac.parser import PACFile

from .scanner import Scanner
from .scanossbase import ScanossBase
from .scanossgrpc import ScanossGrpc


class Components(ScanossBase):
    """
    Class for Component functionality
    """

    def __init__(  # noqa: PLR0913, PLR0915
        self,
        debug: bool = False,
        trace: bool = False,
        quiet: bool = False,
        grpc_url: str = None,
        api_key: str = None,
        timeout: int = 600,
        proxy: str = None,
        grpc_proxy: str = None,
        ca_cert: str = None,
        pac: PACFile = None,
        req_headers: dict = None,
    ):
        """
        Handle all component style requests

        :param debug: Debug
        :param trace: Trace
        :param quiet: Quiet
        :param grpc_url: gRPC URL
        :param api_key: API Key
        :param timeout: Timeout for requests (default 600)
        :param proxy: Proxy to use (optional)
        :param grpc_proxy: Specific gRPC proxy (optional)
        :param ca_cert: TLS client certificate (optional)
        :param pac: Proxy Auto-Config file (optional)
        """
        super().__init__(debug, trace, quiet)
        ver_details = Scanner.version_details()
        self.grpc_api = ScanossGrpc(
            url=grpc_url,
            debug=debug,
            quiet=quiet,
            trace=trace,
            api_key=api_key,
            ver_details=ver_details,
            ca_cert=ca_cert,
            proxy=proxy,
            pac=pac,
            grpc_proxy=grpc_proxy,
            timeout=timeout,
            req_headers=req_headers,
        )

    def load_purls(self, json_file: Optional[str] = None, purls: Optional[List[str]] = None) -> Optional[dict]:
        """
        Load the specified purls and return a dictionary

        :param json_file: JSON PURL file (optional)
        :param purls: list of PURLs (optional)
        :return: PURL Request dictionary or None
        """
        if json_file:
            if not os.path.isfile(json_file) or not os.access(json_file, os.R_OK):
                self.print_stderr(f'ERROR: JSON file does not exist, is not a file, or is not readable: {json_file}')
                return None
            with open(json_file, 'r') as f:
                try:
                    purl_request = json.loads(f.read())
                except Exception as e:
                    self.print_stderr(f'ERROR: Problem parsing input JSON: {e}')
                    return None
        elif purls:
            if not all(isinstance(purl, str) for purl in purls):
                self.print_stderr('ERROR: PURLs must be a list of strings.')
                return None
            parsed_purls = []
            for p in purls:
                parsed_purls.append({'purl': p})
            purl_request = {'purls': parsed_purls}
        else:
            self.print_stderr('ERROR: No purls specified to process.')
            return None
        purl_count = len(purl_request.get('purls', []))
        self.print_debug(f'Parsed Purls ({purl_count}): {purl_request}')
        if purl_count == 0:
            self.print_stderr('ERROR: No PURLs parsed from request.')
            return None
        return purl_request

    def load_json(self, json_file: str = None) -> dict:
        """
        Load the specified json and return a dictionary

        :param json_file: JSON PURL file
        :return: PURL Request dictionary
        """
        if json_file:
            if not os.path.isfile(json_file) or not os.access(json_file, os.R_OK):
                self.print_stderr(f'ERROR: JSON file does not exist, is not a file, or is not readable: {json_file}')
                return None
            with open(json_file, 'r') as f:
                try:
                    return json.loads(f.read())
                except Exception as e:
                    self.print_stderr(f'ERROR: Problem parsing input JSON: {e}')
        return None

    def _open_file_or_sdtout(self, filename):
        """
        Open the given filename if requested, otherwise return STDOUT

        :param filename:
        :return:
        """
        file = sys.stdout
        if filename:
            try:
                file = open(filename, 'w')
            except OSError as e:
                self.print_stderr(f'ERROR: Failed to open output file {filename}: {e}')
                return None
        return file

    def _close_file(self, filename: str = None, file: TextIO = None) -> None:
        """
        Close the file descriptor if its defined

        :param filename: filename
        :param file: file IO object
        :return: None
        """
        if filename and file:
            self.print_trace(f'Closing file: {filename}')
            file.close()

    def get_crypto_details(self, json_file: str = None, purls: [] = None, output_file: str = None) -> bool:
        """
        Retrieve the cryptographic details for the supplied PURLs

        :param json_file: PURL JSON request file (optional)
        :param purls: PURL request array (optional)
        :param output_file: output filename (optional). Default: STDOUT
        :return: True on success, False otherwise
        """
        success = False
        purls_request = self.load_purls(json_file, purls)
        if purls_request is None or len(purls_request) == 0:
            return False
        file = self._open_file_or_sdtout(output_file)
        if file is None:
            return False
        self.print_msg('Sending PURLs to Crypto API for decoration...')
        response = self.grpc_api.get_crypto_json(purls_request)
        if response:
            print(json.dumps(response, indent=2, sort_keys=True), file=file)
            success = True
            if output_file:
                self.print_msg(f'Results written to: {output_file}')
        self._close_file(output_file, file)
        return success

    def get_vulnerabilities(self, json_file: str = None, purls: [] = None, output_file: str = None) -> bool:
        """
        Retrieve any vulnerabilities related to the given PURLs

        :param json_file: PURL JSON request file (optional)
        :param purls: PURL request array (optional)
        :param output_file: output filename (optional). Default: STDOUT
        :return: True on success, False otherwise
        """
        success = False
        purls_request = self.load_purls(json_file, purls)
        if purls_request is None or len(purls_request) == 0:
            return False
        file = self._open_file_or_sdtout(output_file)
        if file is None:
            return False
        self.print_msg('Sending PURLs to Vulnerability API for decoration...')
        response = self.grpc_api.get_vulnerabilities_json(purls_request)
        if response:
            print(json.dumps(response, indent=2, sort_keys=True), file=file)
            success = True
            if output_file:
                self.print_msg(f'Results written to: {output_file}')
        self._close_file(output_file, file)
        return success

    def get_semgrep_details(self, json_file: str = None, purls: [] = None, output_file: str = None) -> bool:
        """
        Retrieve the semgrep details for the supplied PURLs

        :param json_file: PURL JSON request file (optional)
        :param purls: PURL request array (optional)
        :param output_file: output filename (optional). Default: STDOUT
        :return: True on success, False otherwise
        """
        success = False
        purls_request = self.load_purls(json_file, purls)
        if purls_request is None or len(purls_request) == 0:
            return False
        file = self._open_file_or_sdtout(output_file)
        if file is None:
            return False
        self.print_msg('Sending PURLs to Semgrep API for decoration...')
        response = self.grpc_api.get_semgrep_json(purls_request)
        if response:
            print(json.dumps(response, indent=2, sort_keys=True), file=file)
            success = True
            if output_file:
                self.print_msg(f'Results written to: {output_file}')
        self._close_file(output_file, file)
        return success

    def search_components(  # noqa: PLR0913, PLR0915
        self,
        output_file: str = None,
        json_file: str = None,
        search: str = None,
        vendor: str = None,
        comp: str = None,
        package: str = None,
        limit: int = None,
        offset: int = None,
    ) -> bool:
        """
        Search for a component based on the given search criteria

        :param output_file: output filename (optional). Default: STDOUT
        :param json_file: Search JSON request file (optional)
        :param search: Search for (vendor/component/purl) for a component (overrides vendor/component)
        :param vendor: Vendor to search for
        :param comp: Component to search for
        :param package: Package (purl type) to search for. i.e. github/maven/maven/npn/all - default github
        :param limit: Number of matches to return
        :param offset: Offset to submit to return next (limit) of component matches
        :return: True on success, False otherwise
        """
        success = False
        request: dict
        if json_file:  # Parse the json file to extract the search details
            request = self.load_json(json_file)
            if request is None:
                return False
        else:  # Construct a query dictionary from parameters
            request = {'search': search, 'vendor': vendor, 'component': comp, 'package': package}
            if limit is not None and limit > 0:
                request['limit'] = limit
            if offset is not None and offset > 0:
                request['offset'] = offset

        file = self._open_file_or_sdtout(output_file)
        if file is None:
            return False
        self.print_msg('Sending search data to Components API...')
        response = self.grpc_api.search_components_json(request)
        if response:
            print(json.dumps(response, indent=2, sort_keys=True), file=file)
            success = True
            if output_file:
                self.print_msg(f'Results written to: {output_file}')
        self._close_file(output_file, file)
        return success

    def get_component_versions(
        self, output_file: str = None, json_file: str = None, purl: str = None, limit: int = None
    ) -> bool:
        """
        Search for a component versions based on the given search criteria

        :param output_file: output filename (optional). Default: STDOUT
        :param json_file: Search JSON request file (optional)
        :param purl: PURL to retrieve versions for
        :param limit: Number of version to return
        :return: True on success, False otherwise
        """
        success = False
        request: dict
        if json_file:  # Parse the json file to extract the search details
            request = self.load_json(json_file)
            if request is None:
                return False
        else:  # Construct a query dictionary from parameters
            request = {'purl': purl}
            if limit is not None and limit > 0:
                request['limit'] = limit

        file = self._open_file_or_sdtout(output_file)
        if file is None:
            return False
        self.print_msg('Sending PURLs to Component Versions API...')
        response = self.grpc_api.get_component_versions_json(request)
        if response:
            print(json.dumps(response, indent=2, sort_keys=True), file=file)
            success = True
            if output_file:
                self.print_msg(f'Results written to: {output_file}')
        self._close_file(output_file, file)
        return success

    def get_provenance_details(
        self, json_file: str = None, purls: [] = None, output_file: str = None, origin: bool = False
    ) -> bool:
        """
        Retrieve the provenance details for the supplied PURLs

        Args:
            json_file (str, optional): Input JSON file. Defaults to None.
            purls (None, optional): PURLs to retrieve provenance details for. Defaults to None.
            output_file (str, optional): Output file. Defaults to None.
            origin (bool, optional): Retrieve origin details. Defaults to False.

        Returns:
            bool: True on success, False otherwise
        """
        success = False
        purls_request = self.load_purls(json_file, purls)
        if purls_request is None or len(purls_request) == 0:
            return False
        file = self._open_file_or_sdtout(output_file)
        if file is None:
            return False
        if origin:
            self.print_msg('Sending PURLs to Geo Provenance Origin API for decoration...')
            response = self.grpc_api.get_provenance_origin(purls_request)
        else:
            self.print_msg('Sending PURLs to Geo Provenance Declared API for decoration...')
            response = self.grpc_api.get_provenance_json(purls_request)
        if response:
            print(json.dumps(response, indent=2, sort_keys=True), file=file)
            success = True
            if output_file:
                self.print_msg(f'Results written to: {output_file}')
        self._close_file(output_file, file)
        return success



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from scanoss.api.common.v2 import scanoss_common_pb2 as scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2
from scanoss.api.geoprovenance.v2 import scanoss_geoprovenance_pb2 as scanoss_dot_api_dot_geoprovenance_dot_v2_dot_scanoss__geoprovenance__pb2

GRPC_GENERATED_VERSION = '1.73.1'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in scanoss/api/geoprovenance/v2/scanoss_geoprovenance_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class GeoProvenanceStub(object):
    """*
    Expose all of the SCANOSS Geo Provenance RPCs here
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Echo = channel.unary_unary(
                '/scanoss.api.geoprovenance.v2.GeoProvenance/Echo',
                request_serializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.EchoRequest.SerializeToString,
                response_deserializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.EchoResponse.FromString,
                _registered_method=True)
        self.GetComponentContributors = channel.unary_unary(
                '/scanoss.api.geoprovenance.v2.GeoProvenance/GetComponentContributors',
                request_serializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.PurlRequest.SerializeToString,
                response_deserializer=scanoss_dot_api_dot_geoprovenance_dot_v2_dot_scanoss__geoprovenance__pb2.ContributorResponse.FromString,
                _registered_method=True)
        self.GetComponentOrigin = channel.unary_unary(
                '/scanoss.api.geoprovenance.v2.GeoProvenance/GetComponentOrigin',
                request_serializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.PurlRequest.SerializeToString,
                response_deserializer=scanoss_dot_api_dot_geoprovenance_dot_v2_dot_scanoss__geoprovenance__pb2.OriginResponse.FromString,
                _registered_method=True)


class GeoProvenanceServicer(object):
    """*
    Expose all of the SCANOSS Geo Provenance RPCs here
    """

    def Echo(self, request, context):
        """Standard echo
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetComponentContributors(self, request, context):
        """Get component-level Geo Provenance based on contributor declared location
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetComponentOrigin(self, request, context):
        """Get component-level Geo Provenance based on contributor origin commit times
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_GeoProvenanceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Echo': grpc.unary_unary_rpc_method_handler(
                    servicer.Echo,
                    request_deserializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.EchoRequest.FromString,
                    response_serializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.EchoResponse.SerializeToString,
            ),
            'GetComponentContributors': grpc.unary_unary_rpc_method_handler(
                    servicer.GetComponentContributors,
                    request_deserializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.PurlRequest.FromString,
                    response_serializer=scanoss_dot_api_dot_geoprovenance_dot_v2_dot_scanoss__geoprovenance__pb2.ContributorResponse.SerializeToString,
            ),
            'GetComponentOrigin': grpc.unary_unary_rpc_method_handler(
                    servicer.GetComponentOrigin,
                    request_deserializer=scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.PurlRequest.FromString,
                    response_serializer=scanoss_dot_api_dot_geoprovenance_dot_v2_dot_scanoss__geoprovenance__pb2.OriginResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'scanoss.api.geoprovenance.v2.GeoProvenance', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('scanoss.api.geoprovenance.v2.GeoProvenance', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class GeoProvenance(object):
    """*
    Expose all of the SCANOSS Geo Provenance RPCs here
    """

    @staticmethod
    def Echo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/scanoss.api.geoprovenance.v2.GeoProvenance/Echo',
            scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.EchoRequest.SerializeToString,
            scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.EchoResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetComponentContributors(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/scanoss.api.geoprovenance.v2.GeoProvenance/GetComponentContributors',
            scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.PurlRequest.SerializeToString,
            scanoss_dot_api_dot_geoprovenance_dot_v2_dot_scanoss__geoprovenance__pb2.ContributorResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetComponentOrigin(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/scanoss.api.geoprovenance.v2.GeoProvenance/GetComponentOrigin',
            scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2.PurlRequest.SerializeToString,
            scanoss_dot_api_dot_geoprovenance_dot_v2_dot_scanoss__geoprovenance__pb2.OriginResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: scanoss/api/dependencies/v2/scanoss-dependencies.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'scanoss/api/dependencies/v2/scanoss-dependencies.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from scanoss.api.common.v2 import scanoss_common_pb2 as scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from protoc_gen_openapiv2.options import annotations_pb2 as protoc__gen__openapiv2_dot_options_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n6scanoss/api/dependencies/v2/scanoss-dependencies.proto\x12\x1bscanoss.api.dependencies.v2\x1a*scanoss/api/common/v2/scanoss-common.proto\x1a\x1cgoogle/api/annotations.proto\x1a.protoc-gen-openapiv2/options/annotations.proto\"\xef\x01\n\x11\x44\x65pendencyRequest\x12\x43\n\x05\x66iles\x18\x01 \x03(\x0b\x32\x34.scanoss.api.dependencies.v2.DependencyRequest.Files\x12\r\n\x05\x64\x65pth\x18\x02 \x01(\x05\x1a*\n\x05Purls\x12\x0c\n\x04purl\x18\x01 \x01(\t\x12\x13\n\x0brequirement\x18\x02 \x01(\t\x1aZ\n\x05\x46iles\x12\x0c\n\x04\x66ile\x18\x01 \x01(\t\x12\x43\n\x05purls\x18\x02 \x03(\x0b\x32\x34.scanoss.api.dependencies.v2.DependencyRequest.Purls\"\x98\x04\n\x12\x44\x65pendencyResponse\x12\x44\n\x05\x66iles\x18\x01 \x03(\x0b\x32\x35.scanoss.api.dependencies.v2.DependencyResponse.Files\x12\x35\n\x06status\x18\x02 \x01(\x0b\x32%.scanoss.api.common.v2.StatusResponse\x1aP\n\x08Licenses\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07spdx_id\x18\x02 \x01(\t\x12\x18\n\x10is_spdx_approved\x18\x03 \x01(\x08\x12\x0b\n\x03url\x18\x04 \x01(\t\x1a\xaa\x01\n\x0c\x44\x65pendencies\x12\x11\n\tcomponent\x18\x01 \x01(\t\x12\x0c\n\x04purl\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\t\x12J\n\x08licenses\x18\x04 \x03(\x0b\x32\x38.scanoss.api.dependencies.v2.DependencyResponse.Licenses\x12\x0b\n\x03url\x18\x05 \x01(\t\x12\x0f\n\x07\x63omment\x18\x06 \x01(\t\x1a\x85\x01\n\x05\x46iles\x12\x0c\n\x04\x66ile\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\t\x12\x0e\n\x06status\x18\x03 \x01(\t\x12R\n\x0c\x64\x65pendencies\x18\x04 \x03(\x0b\x32<.scanoss.api.dependencies.v2.DependencyResponse.Dependencies\"z\n\x1bTransitiveDependencyRequest\x12\x11\n\tecosystem\x18\x01 \x01(\t\x12\r\n\x05\x64\x65pth\x18\x02 \x01(\x05\x12\r\n\x05limit\x18\x03 \x01(\x05\x12*\n\x05purls\x18\x05 \x03(\x0b\x32\x1b.scanoss.api.common.v2.Purl\"\xe2\x01\n\x1cTransitiveDependencyResponse\x12\\\n\x0c\x64\x65pendencies\x18\x01 \x03(\x0b\x32\x46.scanoss.api.dependencies.v2.TransitiveDependencyResponse.Dependencies\x12\x35\n\x06status\x18\x02 \x01(\x0b\x32%.scanoss.api.common.v2.StatusResponse\x1a-\n\x0c\x44\x65pendencies\x12\x0c\n\x04purl\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t2\xe7\x03\n\x0c\x44\x65pendencies\x12u\n\x04\x45\x63ho\x12\".scanoss.api.common.v2.EchoRequest\x1a#.scanoss.api.common.v2.EchoResponse\"$\x82\xd3\xe4\x93\x02\x1e\"\x19/api/v2/dependencies/echo:\x01*\x12\xa0\x01\n\x0fGetDependencies\x12..scanoss.api.dependencies.v2.DependencyRequest\x1a/.scanoss.api.dependencies.v2.DependencyResponse\",\x82\xd3\xe4\x93\x02&\"!/api/v2/dependencies/dependencies:\x01*\x12\xbc\x01\n\x19GetTransitiveDependencies\x12\x38.scanoss.api.dependencies.v2.TransitiveDependencyRequest\x1a\x39.scanoss.api.dependencies.v2.TransitiveDependencyResponse\"*\x82\xd3\xe4\x93\x02$\"\x1f/api/v2/dependencies/transitive:\x01*B\x9c\x02Z9github.com/scanoss/papi/api/dependenciesv2;dependenciesv2\x92\x41\xdd\x01\x12w\n\x1aSCANOSS Dependency Service\"T\n\x14scanoss-dependencies\x12\'https://github.com/scanoss/dependencies\x1a\x13support@scanoss.com2\x03\x32.0*\x01\x01\x32\x10\x61pplication/json:\x10\x61pplication/jsonR;\n\x03\x34\x30\x34\x12\x34\n*Returned when the resource does not exist.\x12\x06\n\x04\x9a\x02\x01\x07\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'scanoss.api.dependencies.v2.scanoss_dependencies_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z9github.com/scanoss/papi/api/dependenciesv2;dependenciesv2\222A\335\001\022w\n\032SCANOSS Dependency Service\"T\n\024scanoss-dependencies\022\'https://github.com/scanoss/dependencies\032\023support@scanoss.com2\0032.0*\001\0012\020application/json:\020application/jsonR;\n\003404\0224\n*Returned when the resource does not exist.\022\006\n\004\232\002\001\007'
  _globals['_DEPENDENCIES'].methods_by_name['Echo']._loaded_options = None
  _globals['_DEPENDENCIES'].methods_by_name['Echo']._serialized_options = b'\202\323\344\223\002\036\"\031/api/v2/dependencies/echo:\001*'
  _globals['_DEPENDENCIES'].methods_by_name['GetDependencies']._loaded_options = None
  _globals['_DEPENDENCIES'].methods_by_name['GetDependencies']._serialized_options = b'\202\323\344\223\002&\"!/api/v2/dependencies/dependencies:\001*'
  _globals['_DEPENDENCIES'].methods_by_name['GetTransitiveDependencies']._loaded_options = None
  _globals['_DEPENDENCIES'].methods_by_name['GetTransitiveDependencies']._serialized_options = b'\202\323\344\223\002$\"\037/api/v2/dependencies/transitive:\001*'
  _globals['_DEPENDENCYREQUEST']._serialized_start=210
  _globals['_DEPENDENCYREQUEST']._serialized_end=449
  _globals['_DEPENDENCYREQUEST_PURLS']._serialized_start=315
  _globals['_DEPENDENCYREQUEST_PURLS']._serialized_end=357
  _globals['_DEPENDENCYREQUEST_FILES']._serialized_start=359
  _globals['_DEPENDENCYREQUEST_FILES']._serialized_end=449
  _globals['_DEPENDENCYRESPONSE']._serialized_start=452
  _globals['_DEPENDENCYRESPONSE']._serialized_end=988
  _globals['_DEPENDENCYRESPONSE_LICENSES']._serialized_start=599
  _globals['_DEPENDENCYRESPONSE_LICENSES']._serialized_end=679
  _globals['_DEPENDENCYRESPONSE_DEPENDENCIES']._serialized_start=682
  _globals['_DEPENDENCYRESPONSE_DEPENDENCIES']._serialized_end=852
  _globals['_DEPENDENCYRESPONSE_FILES']._serialized_start=855
  _globals['_DEPENDENCYRESPONSE_FILES']._serialized_end=988
  _globals['_TRANSITIVEDEPENDENCYREQUEST']._serialized_start=990
  _globals['_TRANSITIVEDEPENDENCYREQUEST']._serialized_end=1112
  _globals['_TRANSITIVEDEPENDENCYRESPONSE']._serialized_start=1115
  _globals['_TRANSITIVEDEPENDENCYRESPONSE']._serialized_end=1341
  _globals['_TRANSITIVEDEPENDENCYRESPONSE_DEPENDENCIES']._serialized_start=1296
  _globals['_TRANSITIVEDEPENDENCYRESPONSE_DEPENDENCIES']._serialized_end=1341
  _globals['_DEPENDENCIES']._serialized_start=1344
  _globals['_DEPENDENCIES']._serialized_end=1831
# @@protoc_insertion_point(module_scope)



"""
SPDX-License-Identifier: MIT

  Copyright (c) 2025, SCANOSS

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
"""

import json

from .raw_base import RawBase


class LicenseSummary(RawBase):
    """
       SCANOSS LicenseSummary class
       Inspects results and generates comprehensive license summaries from detected components.

       This class processes component scan results to extract, validate, and aggregate license
       information, providing detailed summaries including copyleft analysis and license statistics.
       """

    # Define required license fields as class constants
    REQUIRED_LICENSE_FIELDS = ['spdxid', 'url', 'copyleft', 'source']

    def __init__( # noqa: PLR0913
        self,
        debug: bool = False,
        trace: bool = False,
        quiet: bool = False,
        filepath: str = None,
        status: str = None,
        output: str = None,
        include: str = None,
        exclude: str = None,
        explicit: str = None,
    ):
        """
        Initialize the LicenseSummary class.

        :param debug: Enable debug mode
        :param trace: Enable trace mode (default True)
        :param quiet: Enable quiet mode
        :param filepath: Path to the file containing component data
        :param output: Path to save detailed output
        :param include: Licenses to include in the analysis
        :param exclude: Licenses to exclude from the analysis
        :param explicit: Explicitly defined licenses
        """
        super().__init__(debug, trace, quiet, filepath = filepath, output=output)
        self.license_util.init(include, exclude, explicit)
        self.filepath = filepath
        self.output = output
        self.status = status
        self.include = include
        self.exclude = exclude
        self.explicit = explicit

    def _get_licenses_summary_from_components(self, components: list)-> dict:
        """
        Get a license summary from detected components.

        :param components: List of all components
        :return: Dict with license summary information
        """
        # A component is considered unique by its combination of PURL (Package URL) and license
        component_licenses = self._group_components_by_license(components)
        license_component_count = {}
        # Count license per component
        for lic in component_licenses:
            if lic['spdxid'] not in license_component_count:
                license_component_count[lic['spdxid']] = 1
            else:
                license_component_count[lic['spdxid']] += 1
        licenses:dict = {}
        for comp_lic in component_licenses:
            spdxid = comp_lic.get("spdxid")
            url = comp_lic.get("url")
            copyleft = comp_lic.get("copyleft")
            if spdxid not in licenses:
                licenses[spdxid] = {
                    'spdxid': spdxid,
                    'url': url,
                    'copyleft': copyleft,
                    'componentCount': license_component_count.get(spdxid, 0), # Append component count to license
                }
            ## End for loop licenses
        ## End for loop components
        detected_licenses = list(licenses.values())
        licenses_with_copyleft = [lic for lic in detected_licenses if lic['copyleft']]
        return {
                 'licenses': detected_licenses,
                 'detectedLicenses': len(detected_licenses), # Count unique licenses. SPDXID is considered unique
                 'detectedLicensesWithCopyleft': len(licenses_with_copyleft),
               }


    def _get_components(self):
        """
        Extract and process components from results and their dependencies.

        This method performs the following steps:
        1. Validates that `self.results` is loaded. Returns `None` if not.
        2. Extracts file, snippet, and dependency components into a dictionary.
        3. Converts components to a list and processes their licenses.

        :return: A list of processed components with license data, or `None` if `self.results` is not set.
        """
        if self.results is None:
            raise ValueError(f'Error: No results found in ${self.filepath}')

        components: dict = {}
        # Extract component and license data from file and dependency results. Both helpers mutate `components`
        self._get_components_data(self.results, components)
        self._get_dependencies_data(self.results, components)
        return self._convert_components_to_list(components)

    def run(self):
        components = self._get_components()
        license_summary = self._get_licenses_summary_from_components(components)
        self.print_to_file_or_stdout(json.dumps(license_summary, indent=2), self.output)
        return license_summary
#
# End of LicenseSummary Class
#


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: scanoss/api/vulnerabilities/v2/scanoss-vulnerabilities.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'scanoss/api/vulnerabilities/v2/scanoss-vulnerabilities.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from scanoss.api.common.v2 import scanoss_common_pb2 as scanoss_dot_api_dot_common_dot_v2_dot_scanoss__common__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from protoc_gen_openapiv2.options import annotations_pb2 as protoc__gen__openapiv2_dot_options_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n<scanoss/api/vulnerabilities/v2/scanoss-vulnerabilities.proto\x12\x1escanoss.api.vulnerabilities.v2\x1a*scanoss/api/common/v2/scanoss-common.proto\x1a\x1cgoogle/api/annotations.proto\x1a.protoc-gen-openapiv2/options/annotations.proto\"\x91\x01\n\x14VulnerabilityRequest\x12I\n\x05purls\x18\x01 \x03(\x0b\x32:.scanoss.api.vulnerabilities.v2.VulnerabilityRequest.Purls\x1a*\n\x05Purls\x12\x0c\n\x04purl\x18\x01 \x01(\t\x12\x13\n\x0brequirement\x18\x02 \x01(\t:\x02\x18\x01\"\xaf\x01\n\x0b\x43peResponse\x12@\n\x05purls\x18\x01 \x03(\x0b\x32\x31.scanoss.api.vulnerabilities.v2.CpeResponse.Purls\x12\x35\n\x06status\x18\x02 \x01(\x0b\x32%.scanoss.api.common.v2.StatusResponse\x1a#\n\x05Purls\x12\x0c\n\x04purl\x18\x01 \x01(\t\x12\x0c\n\x04\x63pes\x18\x02 \x03(\t:\x02\x18\x01\"U\n\x11\x43omponentCpesInfo\x12\x0c\n\x04purl\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x13\n\x0brequirement\x18\x03 \x01(\t\x12\x0c\n\x04\x63pes\x18\x04 \x03(\t\"\x8b\x03\n\x15\x43omponentCpesResponse\x12\x44\n\tcomponent\x18\x01 \x01(\x0b\x32\x31.scanoss.api.vulnerabilities.v2.ComponentCpesInfo\x12\x35\n\x06status\x18\x02 \x01(\x0b\x32%.scanoss.api.common.v2.StatusResponse:\xf4\x01\x92\x41\xf0\x01\n\xed\x01J\xea\x01{\"component\":{\"purl\": \"pkg:github/scanoss/engine@1.0.0\", \"requirement\": \"1.0.0\", \"version\": \"1.0.0\", \"cpes\": [\"cpe:2.3:a:scanoss:engine:1.0.0:*:*:*:*:*:*:*\"]}, \"status\": {\"status\": \"SUCCESS\", \"message\": \"CPEs Successfully retrieved\"}}\"\xa7\x04\n\x16\x43omponentsCpesResponse\x12\x45\n\ncomponents\x18\x01 \x03(\x0b\x32\x31.scanoss.api.vulnerabilities.v2.ComponentCpesInfo\x12\x35\n\x06status\x18\x02 \x01(\x0b\x32%.scanoss.api.common.v2.StatusResponse:\x8e\x03\x92\x41\x8a\x03\n\x87\x03J\x84\x03{\"components\":[{\"purl\": \"pkg:github/scanoss/engine\", \"requirement=\": \"1.0.0\", \"version=\": \"1.0.0\", \"cpes\": [\"cpe:2.3:a:scanoss:engine:1.0.0:*:*:*:*:*:*:*\"]}, {\"purl\": \"pkg:github/scanoss/scanoss.py@v1.30.0\",\"requirement\": \"\",\"version\": \"v1.30.0\", \"cpes\": [\"cpe:2.3:a:scanoss:scanoss.py:1.30.0:*:*:*:*:*:*:*\"]}  ], \"status\": {\"status\": \"SUCCESS\", \"message\": \"CPEs Successfully retrieved\"}}\"?\n\x04\x43VSS\x12\x0c\n\x04\x63vss\x18\x01 \x01(\t\x12\x12\n\ncvss_score\x18\x02 \x01(\x02\x12\x15\n\rcvss_severity\x18\x03 \x01(\t\"\xc1\x01\n\rVulnerability\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0b\n\x03\x63ve\x18\x02 \x01(\t\x12\x0b\n\x03url\x18\x03 \x01(\t\x12\x0f\n\x07summary\x18\x04 \x01(\t\x12\x10\n\x08severity\x18\x05 \x01(\t\x12\x11\n\tpublished\x18\x06 \x01(\t\x12\x10\n\x08modified\x18\x07 \x01(\t\x12\x0e\n\x06source\x18\x08 \x01(\t\x12\x32\n\x04\x63vss\x18\t \x03(\x0b\x32$.scanoss.api.vulnerabilities.v2.CVSS\"\xfd\x01\n\x15VulnerabilityResponse\x12J\n\x05purls\x18\x01 \x03(\x0b\x32;.scanoss.api.vulnerabilities.v2.VulnerabilityResponse.Purls\x12\x35\n\x06status\x18\x02 \x01(\x0b\x32%.scanoss.api.common.v2.StatusResponse\x1a]\n\x05Purls\x12\x0c\n\x04purl\x18\x01 \x01(\t\x12\x46\n\x0fvulnerabilities\x18\x02 \x03(\x0b\x32-.scanoss.api.vulnerabilities.v2.Vulnerability:\x02\x18\x01\"\x98\x01\n\x1a\x43omponentVulnerabilityInfo\x12\x0c\n\x04purl\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x13\n\x0brequirement\x18\x03 \x01(\t\x12\x46\n\x0fvulnerabilities\x18\x04 \x03(\x0b\x32-.scanoss.api.vulnerabilities.v2.Vulnerability\"\xe3\x05\n\x1e\x43omponentVulnerabilityResponse\x12M\n\tcomponent\x18\x01 \x01(\x0b\x32:.scanoss.api.vulnerabilities.v2.ComponentVulnerabilityInfo\x12\x35\n\x06status\x18\x02 \x01(\x0b\x32%.scanoss.api.common.v2.StatusResponse:\xba\x04\x92\x41\xb6\x04\n\xb3\x04J\xb0\x04{\"component\":{\"purl\": \"pkg:github/scanoss/engine\", \"requirement\": \"=>1.0.0\", \"version\": \"1.0.0\", \"vulnerabilities\": [{\"id\": \"DLA-2640-1\", \"cve\": \"DLA-2640-1\", \"url\": \"https://osv.dev/vulnerability/DLA-2640-1\", \"summary\": \"gst-plugins-good1.0 - security update\", \"severity\": \"Critical\", \"published\": \"2021-04-26\", \"modified\": \"2025-05-26\", \"source\": \"OSV\", \"cvss\": [{\"cvss\": \"CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H\", \"cvss_score\": 9.8, \"cvss_severity\": \"Critical\"}]}]}, \"status\": {\"status\": \"SUCCESS\", \"message\": \"Vulnerabilities Successfully retrieved\"}}\"\xbd\t\n\x1f\x43omponentsVulnerabilityResponse\x12N\n\ncomponents\x18\x01 \x03(\x0b\x32:.scanoss.api.vulnerabilities.v2.ComponentVulnerabilityInfo\x12\x35\n\x06status\x18\x02 \x01(\x0b\x32%.scanoss.api.common.v2.StatusResponse:\x92\x08\x92\x41\x8e\x08\n\x8b\x08J\x88\x08{\"components\":[{\"purl\": \"pkg:github/scanoss/engine\", \"requirement\": \"1.0.0\", \"version\": \"1.0.0\", \"vulnerabilities\": [{\"id\": \"DLA-2640-1\", \"cve\": \"DLA-2640-1\", \"url\": \"https://osv.dev/vulnerability/DLA-2640-1\", \"summary\": \"gst-plugins-good1.0 - security update\", \"severity\": \"Critical\", \"published\": \"2021-04-26\", \"modified\": \"2025-05-26\", \"source\": \"OSV\", \"cvss\": [{\"cvss\": \"CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H\", \"cvss_score\": 9.8, \"cvss_severity\": \"Critical\"}]}]}, {\"purl\": \"pkg:github/scanoss/scanoss.py\",\"requirement\": \"v1.30.0\",\"version\": \"v1.30.0\", \"vulnerabilities\": [{\"id\": \"CVE-2024-54321\", \"cve\": \"CVE-2024-54321\", \"url\": \"https://nvd.nist.gov/vuln/detail/CVE-2024-54321\", \"summary\": \"Denial of service vulnerability\", \"severity\": \"Medium\", \"published\": \"2024-01-15\", \"modified\": \"2024-02-01\", \"source\": \"NDV\", \"cvss\": [{\"cvss\": \"CVSS:3.1/AV:N/AC:L/PR:N/UI:R/S:U/C:N/I:N/A:L\", \"cvss_score\": 4.3, \"cvss_severity\": \"Medium\"}]}]}], \"status\": {\"status\": \"SUCCESS\", \"message\": \"Vulnerabilities Successfully retrieved\"}}2\xc7\x08\n\x0fVulnerabilities\x12x\n\x04\x45\x63ho\x12\".scanoss.api.common.v2.EchoRequest\x1a#.scanoss.api.common.v2.EchoResponse\"\'\x82\xd3\xe4\x93\x02!\"\x1c/api/v2/vulnerabilities/echo:\x01*\x12q\n\x07GetCpes\x12\x34.scanoss.api.vulnerabilities.v2.VulnerabilityRequest\x1a+.scanoss.api.vulnerabilities.v2.CpeResponse\"\x03\x88\x02\x01\x12\xa2\x01\n\x10GetComponentCpes\x12\'.scanoss.api.common.v2.ComponentRequest\x1a\x35.scanoss.api.vulnerabilities.v2.ComponentCpesResponse\".\x82\xd3\xe4\x93\x02(\x12&/api/v2/vulnerabilities/cpes/component\x12\xa9\x01\n\x11GetComponentsCpes\x12(.scanoss.api.common.v2.ComponentsRequest\x1a\x36.scanoss.api.vulnerabilities.v2.ComponentsCpesResponse\"2\x82\xd3\xe4\x93\x02,\"\'/api/v2/vulnerabilities/cpes/components:\x01*\x12\x86\x01\n\x12GetVulnerabilities\x12\x34.scanoss.api.vulnerabilities.v2.VulnerabilityRequest\x1a\x35.scanoss.api.vulnerabilities.v2.VulnerabilityResponse\"\x03\x88\x02\x01\x12\xb1\x01\n\x1bGetComponentVulnerabilities\x12\'.scanoss.api.common.v2.ComponentRequest\x1a>.scanoss.api.vulnerabilities.v2.ComponentVulnerabilityResponse\")\x82\xd3\xe4\x93\x02#\x12!/api/v2/vulnerabilities/component\x12\xb8\x01\n\x1cGetComponentsVulnerabilities\x12(.scanoss.api.common.v2.ComponentsRequest\x1a?.scanoss.api.vulnerabilities.v2.ComponentsVulnerabilityResponse\"-\x82\xd3\xe4\x93\x02\'\"\"/api/v2/vulnerabilities/components:\x01*B\x92\x03Z?github.com/scanoss/papi/api/vulnerabilitiesv2;vulnerabilitiesv2\x92\x41\xcd\x02\x12\xd4\x01\n\x1dSCANOSS Vulnerability Service\x12RVulnerability service provides vulnerability intelligence for software components.\"Z\n\x17scanoss-vulnerabilities\x12*https://github.com/scanoss/vulnerabilities\x1a\x13support@scanoss.com2\x03\x32.0\x1a\x0f\x61pi.scanoss.com*\x02\x01\x02\x32\x10\x61pplication/json:\x10\x61pplication/jsonR;\n\x03\x34\x30\x34\x12\x34\n*Returned when the resource does not exist.\x12\x06\n\x04\x9a\x02\x01\x07\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'scanoss.api.vulnerabilities.v2.scanoss_vulnerabilities_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z?github.com/scanoss/papi/api/vulnerabilitiesv2;vulnerabilitiesv2\222A\315\002\022\324\001\n\035SCANOSS Vulnerability Service\022RVulnerability service provides vulnerability intelligence for software components.\"Z\n\027scanoss-vulnerabilities\022*https://github.com/scanoss/vulnerabilities\032\023support@scanoss.com2\0032.0\032\017api.scanoss.com*\002\001\0022\020application/json:\020application/jsonR;\n\003404\0224\n*Returned when the resource does not exist.\022\006\n\004\232\002\001\007'
  _globals['_VULNERABILITYREQUEST']._loaded_options = None
  _globals['_VULNERABILITYREQUEST']._serialized_options = b'\030\001'
  _globals['_CPERESPONSE']._loaded_options = None
  _globals['_CPERESPONSE']._serialized_options = b'\030\001'
  _globals['_COMPONENTCPESRESPONSE']._loaded_options = None
  _globals['_COMPONENTCPESRESPONSE']._serialized_options = b'\222A\360\001\n\355\001J\352\001{\"component\":{\"purl\": \"pkg:github/scanoss/engine@1.0.0\", \"requirement\": \"1.0.0\", \"version\": \"1.0.0\", \"cpes\": [\"cpe:2.3:a:scanoss:engine:1.0.0:*:*:*:*:*:*:*\"]}, \"status\": {\"status\": \"SUCCESS\", \"message\": \"CPEs Successfully retrieved\"}}'
  _globals['_COMPONENTSCPESRESPONSE']._loaded_options = None
  _globals['_COMPONENTSCPESRESPONSE']._serialized_options = b'\222A\212\003\n\207\003J\204\003{\"components\":[{\"purl\": \"pkg:github/scanoss/engine\", \"requirement=\": \"1.0.0\", \"version=\": \"1.0.0\", \"cpes\": [\"cpe:2.3:a:scanoss:engine:1.0.0:*:*:*:*:*:*:*\"]}, {\"purl\": \"pkg:github/scanoss/scanoss.py@v1.30.0\",\"requirement\": \"\",\"version\": \"v1.30.0\", \"cpes\": [\"cpe:2.3:a:scanoss:scanoss.py:1.30.0:*:*:*:*:*:*:*\"]}  ], \"status\": {\"status\": \"SUCCESS\", \"message\": \"CPEs Successfully retrieved\"}}'
  _globals['_VULNERABILITYRESPONSE']._loaded_options = None
  _globals['_VULNERABILITYRESPONSE']._serialized_options = b'\030\001'
  _globals['_COMPONENTVULNERABILITYRESPONSE']._loaded_options = None
  _globals['_COMPONENTVULNERABILITYRESPONSE']._serialized_options = b'\222A\266\004\n\263\004J\260\004{\"component\":{\"purl\": \"pkg:github/scanoss/engine\", \"requirement\": \"=>1.0.0\", \"version\": \"1.0.0\", \"vulnerabilities\": [{\"id\": \"DLA-2640-1\", \"cve\": \"DLA-2640-1\", \"url\": \"https://osv.dev/vulnerability/DLA-2640-1\", \"summary\": \"gst-plugins-good1.0 - security update\", \"severity\": \"Critical\", \"published\": \"2021-04-26\", \"modified\": \"2025-05-26\", \"source\": \"OSV\", \"cvss\": [{\"cvss\": \"CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H\", \"cvss_score\": 9.8, \"cvss_severity\": \"Critical\"}]}]}, \"status\": {\"status\": \"SUCCESS\", \"message\": \"Vulnerabilities Successfully retrieved\"}}'
  _globals['_COMPONENTSVULNERABILITYRESPONSE']._loaded_options = None
  _globals['_COMPONENTSVULNERABILITYRESPONSE']._serialized_options = b'\222A\216\010\n\213\010J\210\010{\"components\":[{\"purl\": \"pkg:github/scanoss/engine\", \"requirement\": \"1.0.0\", \"version\": \"1.0.0\", \"vulnerabilities\": [{\"id\": \"DLA-2640-1\", \"cve\": \"DLA-2640-1\", \"url\": \"https://osv.dev/vulnerability/DLA-2640-1\", \"summary\": \"gst-plugins-good1.0 - security update\", \"severity\": \"Critical\", \"published\": \"2021-04-26\", \"modified\": \"2025-05-26\", \"source\": \"OSV\", \"cvss\": [{\"cvss\": \"CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H\", \"cvss_score\": 9.8, \"cvss_severity\": \"Critical\"}]}]}, {\"purl\": \"pkg:github/scanoss/scanoss.py\",\"requirement\": \"v1.30.0\",\"version\": \"v1.30.0\", \"vulnerabilities\": [{\"id\": \"CVE-2024-54321\", \"cve\": \"CVE-2024-54321\", \"url\": \"https://nvd.nist.gov/vuln/detail/CVE-2024-54321\", \"summary\": \"Denial of service vulnerability\", \"severity\": \"Medium\", \"published\": \"2024-01-15\", \"modified\": \"2024-02-01\", \"source\": \"NDV\", \"cvss\": [{\"cvss\": \"CVSS:3.1/AV:N/AC:L/PR:N/UI:R/S:U/C:N/I:N/A:L\", \"cvss_score\": 4.3, \"cvss_severity\": \"Medium\"}]}]}], \"status\": {\"status\": \"SUCCESS\", \"message\": \"Vulnerabilities Successfully retrieved\"}}'
  _globals['_VULNERABILITIES'].methods_by_name['Echo']._loaded_options = None
  _globals['_VULNERABILITIES'].methods_by_name['Echo']._serialized_options = b'\202\323\344\223\002!\"\034/api/v2/vulnerabilities/echo:\001*'
  _globals['_VULNERABILITIES'].methods_by_name['GetCpes']._loaded_options = None
  _globals['_VULNERABILITIES'].methods_by_name['GetCpes']._serialized_options = b'\210\002\001'
  _globals['_VULNERABILITIES'].methods_by_name['GetComponentCpes']._loaded_options = None
  _globals['_VULNERABILITIES'].methods_by_name['GetComponentCpes']._serialized_options = b'\202\323\344\223\002(\022&/api/v2/vulnerabilities/cpes/component'
  _globals['_VULNERABILITIES'].methods_by_name['GetComponentsCpes']._loaded_options = None
  _globals['_VULNERABILITIES'].methods_by_name['GetComponentsCpes']._serialized_options = b'\202\323\344\223\002,\"\'/api/v2/vulnerabilities/cpes/components:\001*'
  _globals['_VULNERABILITIES'].methods_by_name['GetVulnerabilities']._loaded_options = None
  _globals['_VULNERABILITIES'].methods_by_name['GetVulnerabilities']._serialized_options = b'\210\002\001'
  _globals['_VULNERABILITIES'].methods_by_name['GetComponentVulnerabilities']._loaded_options = None
  _globals['_VULNERABILITIES'].methods_by_name['GetComponentVulnerabilities']._serialized_options = b'\202\323\344\223\002#\022!/api/v2/vulnerabilities/component'
  _globals['_VULNERABILITIES'].methods_by_name['GetComponentsVulnerabilities']._loaded_options = None
  _globals['_VULNERABILITIES'].methods_by_name['GetComponentsVulnerabilities']._serialized_options = b'\202\323\344\223\002\'\"\"/api/v2/vulnerabilities/components:\001*'
  _globals['_VULNERABILITYREQUEST']._serialized_start=219
  _globals['_VULNERABILITYREQUEST']._serialized_end=364
  _globals['_VULNERABILITYREQUEST_PURLS']._serialized_start=318
  _globals['_VULNERABILITYREQUEST_PURLS']._serialized_end=360
  _globals['_CPERESPONSE']._serialized_start=367
  _globals['_CPERESPONSE']._serialized_end=542
  _globals['_CPERESPONSE_PURLS']._serialized_start=503
  _globals['_CPERESPONSE_PURLS']._serialized_end=538
  _globals['_COMPONENTCPESINFO']._serialized_start=544
  _globals['_COMPONENTCPESINFO']._serialized_end=629
  _globals['_COMPONENTCPESRESPONSE']._serialized_start=632
  _globals['_COMPONENTCPESRESPONSE']._serialized_end=1027
  _globals['_COMPONENTSCPESRESPONSE']._serialized_start=1030
  _globals['_COMPONENTSCPESRESPONSE']._serialized_end=1581
  _globals['_CVSS']._serialized_start=1583
  _globals['_CVSS']._serialized_end=1646
  _globals['_VULNERABILITY']._serialized_start=1649
  _globals['_VULNERABILITY']._serialized_end=1842
  _globals['_VULNERABILITYRESPONSE']._serialized_start=1845
  _globals['_VULNERABILITYRESPONSE']._serialized_end=2098
  _globals['_VULNERABILITYRESPONSE_PURLS']._serialized_start=2001
  _globals['_VULNERABILITYRESPONSE_PURLS']._serialized_end=2094
  _globals['_COMPONENTVULNERABILITYINFO']._serialized_start=2101
  _globals['_COMPONENTVULNERABILITYINFO']._serialized_end=2253
  _globals['_COMPONENTVULNERABILITYRESPONSE']._serialized_start=2256
  _globals['_COMPONENTVULNERABILITYRESPONSE']._serialized_end=2995
  _globals['_COMPONENTSVULNERABILITYRESPONSE']._serialized_start=2998
  _globals['_COMPONENTSVULNERABILITYRESPONSE']._serialized_end=4211
  _globals['_VULNERABILITIES']._serialized_start=4214
  _globals['_VULNERABILITIES']._serialized_end=5309
# @@protoc_insertion_point(module_scope)
